unit uPlannerRole;

interface

uses
  System.SysUtils, System.Generics.Collections, System.DateUtils,
  Vcl.Dialogs, Vcl.Forms, Vcl.ComCtrls,
  FireDAC.Comp.Client,
  Data.DB,
  AdvOfficePager, AdvSpin, AdvGlowButton, AdvSmoothDatePicker, AdvOfficeButtons, AdvPanel, AdvCombo,
  DBPlanner, Planner, PlanDraw;

type
  TCommon = class(TObject)
  strict private
    FColumnWidth: TAdvSpinEdit;
    FDisplayScale: TAdvSpinEdit;
    FHeaderHeight: TAdvSpinEdit;
    FInPositionGap: TAdvOfficeCheckBox;
    FNumberOfResource: TAdvSpinEdit;
    FPropertiePanel: TAdvPanel;
    FRepeatSideBar: TAdvOfficeCheckBox;
    FShowOccupied: TAdvOfficeCheckBox;
    FSideBarPosition: TAdvOfficeRadioGroup;
    FSideBarWidth: TAdvSpinEdit;
  public
    property ColumnWidth: TAdvSpinEdit             read FColumnWidth      write FColumnWidth;
    property DisplayScale: TAdvSpinEdit            read FDisplayScale     write FDisplayScale;
    property HeaderHeight: TAdvSpinEdit            read FHeaderHeight     write FHeaderHeight;
    property InPositionGap: TAdvOfficeCheckBox     read FInPositionGap    write FInPositionGap;
    property NumberOfResource: TAdvSpinEdit        read FNumberOfResource write FNumberOfResource;
    property PropertiePanel: TAdvPanel             read FPropertiePanel   write FPropertiePanel;
    property RepeatSideBar: TAdvOfficeCheckBox     read FRepeatSideBar    write FRepeatSideBar;
    property ShowOccupied: TAdvOfficeCheckBox      read FShowOccupied     write FShowOccupied;
    property SideBarPosition: TAdvOfficeRadioGroup read FSideBarPosition  write FSideBarPosition;
    property SideBarWidth: TAdvSpinEdit            read FSideBarWidth     write FSideBarWidth;
  end;

  TPeriod = class(TObject)
  strict private
    FButtonAply: TAdvGlowButton;
    FDayFrom: TAdvSmoothDatePicker;
    FDayTo: TAdvSmoothDatePicker;
  public
    property ButtonAply: TAdvGlowButton    read FButtonAply write FButtonAply;
    property DayFrom: TAdvSmoothDatePicker read FDayFrom    write FDayFrom;
    property DayTo: TAdvSmoothDatePicker   read FDayTo      write FDayTo;
  end;

  TTimes = class(TObject)
  strict private
    FActiveEnd: TDateTimePicker;
    FActiveStart: TDateTimePicker;
    FButtonAply: TAdvGlowButton;
    FDisplayEnd: TDateTimePicker;
    FDisplayStart: TDateTimePicker;
    FOnlyActive: TAdvOfficeCheckBox;
    FTimeLine: TAdvComboBox;
  public
    property ActiveEnd: TDateTimePicker         read FActiveEnd    write FActiveEnd;
    property ActiveStart: TDateTimePicker       read FActiveStart  write FActiveStart;
    property ButtonAply: TAdvGlowButton     read FButtonAply   write FButtonAply;
    property DisplayEnd: TDateTimePicker        read FDisplayEnd   write FDisplayEnd;
    property DisplayStart: TDateTimePicker      read FDisplayStart write FDisplayStart;
    property OnlyActive: TAdvOfficeCheckBox read FOnlyActive   write FOnlyActive;
    property TimeLine: TAdvComboBox         read FTimeLine     write FTimeLine;
  end;

  TOther = class(TObject)
  strict private
    FDay: TAdvSmoothDatePicker;
    FIncrement: TAdvSpinEdit;
    FMonth: TAdvComboBox;
    FNumberOf: TAdvSpinEdit;
    FStart: TAdvComboBox;
    FYear: TAdvSpinEdit;
  public
    property Day: TAdvSmoothDatePicker read FDay       write FDay;
    property Increment: TAdvSpinEdit   read FIncrement write FIncrement;
    property Month: TAdvComboBox       read FMonth     write FMonth;
    property NumberOf: TAdvSpinEdit    read FNumberOf  write FNumberOf;
    property Start: TAdvComboBox       read FStart     write FStart;
    property Year: TAdvSpinEdit        read FYear      write FYear;
  end;

  TRolePlanner = class(TObject)
  strict private
  const
    SideBar45 = 45;
    SideBar65 = 65;
    SideBarTop = 2;
  var
    FcbxViews: TAdvComboBox;
    FCommon: TList<TCommon>;
    FDataSet: TDataSet;
    FDBKey: Integer;
    FDisplayUnit: TList<Integer>;
    FForm: TForm;
    FItem: TPlannerItem;
    FOther: TList<TOther>;
    FPager: TAdvOfficePager;
    FPeriod: TList<TPeriod>;
    FPlannerDay: TDBPlanner;
    FPlanners: Integer;
    FPlannerWeek: TDBPlanner;
    FPositionBar: TList<TAdvOfficeRadioGroup>;
    FPropertiePanel: array[0..7] of TAdvPanel;
    FqryCaptionType: TFDQuery;
    FqryItemColor: TFDQuery;
    FqryItemDelete: TFDQuery;
    FqryItemShape: TFDQuery;
    FShapeDrawTool: TObjectList<TShapeDrawTool>;
    FTimeLineUnit: TList<Integer>;
    FTimes: TList<TTimes>;
    procedure CreateListOfComponent;
    procedure CreateObjectSQL;
    procedure FillActiveDay;
    procedure FillActiveDayPeriod;
    procedure FillHalfDayPeriod;
    procedure FillMonth;
    procedure FillMultiDay;
    procedure FillMultiMonth;
    procedure FillPeriod;
    procedure FillPositionBar;
    procedure FillPropertiePanel;
    procedure FillTimeLine;
    procedure GetDisplayUnit;
    procedure GetHourMin(const HourMin: TList<Word>; const I: Integer);
    function GetItem: Boolean;
    procedure GetShapeDrawTool;
    procedure InitialTimeLineUnit;
    procedure SetActiveDay;
    procedure SetActiveDayPeriod;
    procedure SetCommon(const I: Integer);
    procedure SetDisplayStartEnd(const Planner: TDBPlanner);
    procedure SetHalfDayPeriod;
    procedure SetMonth;
    procedure SetMultiDay;
    procedure SetMultiMonth;
    procedure SetNumberOfResources(const I: Integer);
    procedure SetOther(const I: Integer);
    procedure SetPeriod(const I: Integer);
    procedure SetTimeLine(Dsp: TPlannerDisplay; const Hours, I: Integer);
  public
    constructor Create;
    destructor Destroy; override;
    procedure GetHours(var Hours: Integer; const I: Integer);
    procedure ItemShape(const AShape: Byte);
    procedure RepeatSideBar(ARepeat: Boolean);
    procedure SetPlannerProperies;
    procedure SetTimes(const I: Integer);
    property Common: TList<TCommon>                     read FCommon   write FCommon;
    property DisplayUnit: TList<Integer>                read FDisplayUnit;
    property Other: TList<TOther>                       read FOther;
    property Period: TList<TPeriod>                     read FPeriod   write FPeriod;
    property ShapeDrawTool: TObjectList<TShapeDrawTool> read FShapeDrawTool;
    property Times: TList<TTimes>                       read FTimes    write FTimes;
  end;

implementation

{ TRolePlanner }

uses
   PlannerRole;

constructor TRolePlanner.Create;
var
  Frm: TfrmPlannerRole;
begin
  Frm := frmPlannerRole;
  FcbxViews := Frm.advcbxViews;
  CreateObjectSQL;
  CreateListOfComponent;
  InitialTimeLineUnit;
  GetDisplayUnit;
  FPager := Frm.offpgrPlanner;
  FPlannerDay := Frm.dbplDay;
  GetShapeDrawTool;

//  FillPropertiePanel;
//  FillPositionBar;
end;

destructor TRolePlanner.Destroy;
var
  I: Integer;
begin
  for I := 0 to FPlanners - 1 do
    if Assigned(FTimes[I]) then
      FreeAndNil(FTimes[I]);

  for I := 0 to FPlanners - 1 do
    if Assigned(FPeriod[I]) then
      FreeAndNil(FPeriod[I]);

  for I := 0 to FPlanners - 1 do
    if Assigned(FOther[I]) then
      FreeAndNil(FOther[I]);

  for I := 0 to FPlanners - 1 do
    if Assigned(FCommon[I]) then
      FreeAndNil(FCommon[I]);

  FreeAndNil(FTimes);
  FreeAndNil(FPeriod);
  FreeAndNil(FOther);
  FreeAndNil(FCommon);
  FreeAndNil(FDisplayUnit);
  FreeAndNil(FqryCaptionType);
  FreeAndNil(FqryItemColor);
  FreeAndNil(FqryItemDelete);
  FreeAndNil(FqryItemShape);
  FreeAndNil(FShapeDrawTool);
  FreeAndNil(FPositionBar);
  inherited;
end;

procedure TRolePlanner.CreateListOfComponent;
begin
  FTimes := TList<TTimes>.Create;
  FPeriod := TList<TPeriod>.Create;
  FOther := TList<TOther>.Create;
  FCommon := TList<TCommon>.Create;
  FillMultiDay;
  FillPeriod;
  FillHalfDayPeriod;
  FillMonth;
  FillMultiMonth;
  FillTimeLine;
  FillActiveDay;
  FillActiveDayPeriod;
  FPlanners := FTimes.Count;
end;

procedure TRolePlanner.CreateObjectSQL;
begin
Exit;
//  FVstQuery.SQL.Text := TextUpdOrIns('PLANNERS');
//
//  FqryItemColor := TFDFunction.NewQry;
//  FqryItemColor.SQL.Text := 'UPDATE PLANNERS SET COLOR = :COLOR'+
//  ' WHERE IDPLANNERS = :IDPLANNERS';
//
//  FqryItemDelete := TFDFunction.NewQry;
//  FqryItemDelete.SQL.Text := 'DELETE FROM PLANNERS WHERE (IDPLANNERS = :IDPLANNERS)';
//
//  FqryItemShape := TFDFunction.NewQry;
//  FqryItemShape.SQL.Text := 'UPDATE PLANNERS SET SHAPE = :SHAPE'+
//  ' WHERE IDPLANNERS = :IDPLANNERS';
//
//  FqryCaptionType := TFDFunction.NewQry;
//  FqryCaptionType.SQL.Text := 'UPDATE PLANNERS SET CAPTIONTYPE = :CAPTIONTYPE'+
//  ' WHERE IDPLANNERS = :IDPLANNERS';
end;

procedure TRolePlanner.FillActiveDay;
var
  Idx: Integer;
  Frm: TfrmPlannerRole;
begin
  FTimes.Add(TTimes.Create);
  FPeriod.Add(nil);
  FOther.Add(TOther.Create);
  FCommon.Add(TCommon.Create);

  Frm := frmPlannerRole;
  Idx := Ord(TViews(vwActiveDay));
  FTimes[Idx].DisplayStart := Frm.dtpDisplayStartActiveDay;
  FTimes[Idx].DisplayEnd := Frm.dtpDisplayEndActiveDay;
  FTimes[Idx].ActiveStart := Frm.dtpActiveStartActiveDay;
  FTimes[Idx].ActiveEnd := Frm.dtpActiveEndActiveDay;
  FTimes[Idx].TimeLine := Frm.advcbxTimeLineActiveDay;
  FTimes[Idx].ButtonAply := Frm.glwbtnApplyActiveDay;
  FTimes[Idx].ButtonAply.Enabled := False;

  FOther[Idx].Day := Frm.smdtpkDayActiveDay;
  FOther[Idx].NumberOf := Frm.advsedNumberOfDayActiveDay;
  FOther[Idx].Increment := Frm.advsedDayIncrementActiveDay;

  FCommon[Idx].DisplayScale := Frm.advsedRowHeightActiveDay;
  FCommon[Idx].InPositionGap := Frm.offchbInPositionGapActiveDay;
  FCommon[Idx].NumberOfResource := Frm.advsedNumberOfResourceActiveDay;
  FCommon[Idx].PropertiePanel := Frm.jstpnlActiveDay;
  FCommon[Idx].RepeatSideBar := Frm.offchbRepeatSideBarActiveDay;
  FCommon[Idx].ShowOccupied := Frm.offchbShowOccupiedActiveDay;
  FCommon[Idx].SideBarPosition := Frm.offrgrSideBarPositionActiveDay;
  FCommon[Idx].SideBarWidth := Frm.advsedSideBarWidthActiveDay;
  FCommon[Idx].ColumnWidth := Frm.advsedColumnWidthActiveDay;
  FCommon[Idx].HeaderHeight := Frm.advsedHeaderHeightActiveDay;
end;

procedure TRolePlanner.FillActiveDayPeriod;
var
  Idx: Integer;
  Frm: TfrmPlannerRole;
begin
  FTimes.Add(nil);
  FPeriod.Add(TPeriod.Create);
  FOther.Add(nil);
  FCommon.Add(TCommon.Create);

  Frm := frmPlannerRole;
  Idx := Ord(TViews(vwActiveDayPeriod));
  FPeriod[Idx].DayFrom := Frm.smdtpkFromActiveDayPeriod;
  FPeriod[Idx].DayTo := Frm.smdtpkToActiveDayPeriod;
  FPeriod[Idx].ButtonAply := Frm.glwbtnDateActiveDayPeriod;
  FPeriod[Idx].ButtonAply.Enabled := False;

  FCommon[Idx].DisplayScale := Frm.advsedRowHeightActiveDayPeriod;
  FCommon[Idx].InPositionGap := Frm.offchbInPositionGapActiveDayPeriod;
  FCommon[Idx].NumberOfResource := Frm.advsedNumberOfResourceActiveDayPeriod;
  FCommon[Idx].PropertiePanel := Frm.jstpnlActiveDayPeriod;
  FCommon[Idx].ShowOccupied := Frm.offchbShowOccupiedActiveDayPeriod;
  FCommon[Idx].SideBarPosition := Frm.offrgrSideBarPositionActiveDayPeriod;
  FCommon[Idx].SideBarWidth := Frm.advsedSideBarWidthActiveDayPeriod;
  FCommon[Idx].ColumnWidth := Frm.advsedColumnWidthActiveDayPeriod;
  FCommon[Idx].HeaderHeight := Frm.advsedHeaderHeightActiveDayPeriod;
end;

procedure TRolePlanner.FillHalfDayPeriod;
var
  Idx: Integer;
  Frm: TfrmPlannerRole;
begin
  FTimes.Add(nil);
  FPeriod.Add(TPeriod.Create);
  FOther.Add(nil);
  FCommon.Add(TCommon.Create);

  Frm := frmPlannerRole;
  Idx := Ord(TViews(vwHalfDayPeriod));
  FPeriod[Idx].DayFrom := Frm.smdtpkFromHalfDayPeriod;
  FPeriod[Idx].DayTo := Frm.smdtpkToHalfDayPeriod;
  FPeriod[Idx].ButtonAply := Frm.glwbtnPeriodHalfDayPeriod;
  FPeriod[Idx].ButtonAply.Enabled := False;

  FCommon[Idx].DisplayScale := Frm.advsedRowHeightHalfDayPeriod;
  FCommon[Idx].InPositionGap := Frm.offchbInPositionGapHalfDayPeriod;
  FCommon[Idx].NumberOfResource := Frm.advsedNumberOfResourceHalfDayPeriod;
  FCommon[Idx].PropertiePanel := Frm.jstpnlHalfDayPeriod;
  FCommon[Idx].ShowOccupied := Frm.offchbShowOccupiedHalfDayPeriod;
  FCommon[Idx].SideBarPosition := Frm.offrgrSideBarPositionHalfDayPeriod;
  FCommon[Idx].SideBarWidth := Frm.advsedSideBarWidthHalfDayPeriod;
  FCommon[Idx].ColumnWidth := Frm.advsedColumnWidthHalfDayPeriod;
  FCommon[Idx].HeaderHeight := Frm.advsedHeaderHeightHalfDayPeriod;
end;

procedure TRolePlanner.FillMonth;
var
  Idx: Integer;
  Frm: TfrmPlannerRole;
begin
  FTimes.Add(nil);
  FPeriod.Add(nil);
  FOther.Add(TOther.Create);
  FCommon.Add(TCommon.Create);

  Frm := frmPlannerRole;
  Idx := Ord(TViews(vwMonth));
  FOther[Idx].Increment := Frm.advsedMonthIncermentMonth;
  FOther[Idx].Month := Frm.advcbxMonthMonth;
  FOther[Idx].Year := Frm.advsedYearMonth;

  FCommon[Idx].DisplayScale := Frm.advsedRowHeightMonth;
  FCommon[Idx].InPositionGap := Frm.offchbInPositionGapMonth;
  FCommon[Idx].NumberOfResource := Frm.advsedNumberOfResourceMonth;
  FCommon[Idx].PropertiePanel := Frm.jstpnlMonth;
  FCommon[Idx].ShowOccupied := Frm.offchbShowOccupiedMonth;
  FCommon[Idx].SideBarPosition := Frm.offrgrSideBarPositionMonth;
  FCommon[Idx].SideBarWidth := Frm.advsedSideBarWidthMonth;
  FCommon[Idx].ColumnWidth := Frm.advsedColumnWidthMonth;
  FCommon[Idx].HeaderHeight := Frm.advsedHeaderHeightMonth;
end;

procedure TRolePlanner.FillMultiDay;
var
  Idx: Integer;
  Frm: TfrmPlannerRole;
begin
  FTimes.Add(TTimes.Create);
  FPeriod.Add(nil);
  FOther.Add(TOther.Create);
  FCommon.Add(TCommon.Create);

  Frm := frmPlannerRole;
  Idx := Ord(TViews(vwMultiDay));
  FTimes[Idx].DisplayStart := Frm.dtpDisplayStartMultiDay;
  FTimes[Idx].DisplayEnd := Frm.dtpDisplayEndMultiDay;
  FTimes[Idx].ActiveStart := Frm.dtpActiveStartMultiDay;
  FTimes[Idx].ActiveEnd := Frm.dtpActiveEndMultiDay;
  FTimes[Idx].TimeLine := Frm.advcbxTimeLineMultiDay;
  FTimes[Idx].ButtonAply := Frm.glwbtnApplyMultiDay;
  FTimes[Idx].ButtonAply.Enabled := False;

  FOther[Idx].Day := Frm.smdtpkDayMultiDay;
  FOther[Idx].Increment := Frm.advsedDayIncrementMultiDay;
  FOther[Idx].NumberOf := Frm.advsedNumberOfDayMultiDay;

  FCommon[Idx].DisplayScale := Frm.advsedRowHeightMultiDay;
  FCommon[Idx].InPositionGap := Frm.offchbInPositionGapMultiDay;
  FCommon[Idx].NumberOfResource := Frm.advsedNumberOfResourceMultiDay;
  FCommon[Idx].PropertiePanel := Frm.jstpnlMultiDay;
  FCommon[Idx].RepeatSideBar := Frm.offchbRepeatSideBarMultiDay;
  FCommon[Idx].ShowOccupied := Frm.offchbShowOccupiedMultiDay;
  FCommon[Idx].SideBarPosition := Frm.offrgrSideBarPositionMultiDay;
  FCommon[Idx].SideBarWidth := Frm.advsedSideBarWidthMultiDay;
  FCommon[Idx].ColumnWidth := Frm.advsedColumnWidthMultiDay;
  FCommon[Idx].HeaderHeight := Frm.advsedHeaderHeightMultiDay;
end;

procedure TRolePlanner.FillMultiMonth;
var
  Idx: Integer;
  Frm: TfrmPlannerRole;
begin
  FTimes.Add(nil);
  FPeriod.Add(nil);
  FOther.Add(TOther.Create);
  FCommon.Add(TCommon.Create);

  Frm := frmPlannerRole;
  Idx := Ord(TViews(vwMultiMonth));
  FOther[Idx].Increment := Frm.advsedMonthIncrementMultiMonth;
  FOther[Idx].Month := Frm.advcbxStartMonthMultiMonth;
  FOther[Idx].NumberOf := Frm.advsedMonthOfMonthMultiMonth;
  FOther[Idx].Year := Frm.advsedYearMultiMonth;

  FCommon[Idx].DisplayScale := Frm.advsedRowHeightMultiMonth;
  FCommon[Idx].InPositionGap := Frm.offchbInPositionGapMultiMonth;
  FCommon[Idx].PropertiePanel := Frm.jstpnlMultiMonth;
  FCommon[Idx].RepeatSideBar := Frm.offchbRepeatSideBarMultiMonth;
  FCommon[Idx].ShowOccupied := Frm.offchbShowOccupiedMultiMonth;
  FCommon[Idx].SideBarPosition := Frm.offrgrSideBarPositionMultiMonth;
  FCommon[Idx].SideBarWidth := Frm.advsedSideBarWidthMultiMonth;
  FCommon[Idx].ColumnWidth := Frm.advsedColumnWidthMultiMonth;
  FCommon[Idx].HeaderHeight := Frm.advsedHeaderHeightMultiMonth;
end;

procedure TRolePlanner.FillPeriod;
var
  Idx: Integer;
  Frm: TfrmPlannerRole;
begin
  FTimes.Add(nil);
  FPeriod.Add(TPeriod.Create);
  FOther.Add(nil);
  FCommon.Add(TCommon.Create);

  Frm := frmPlannerRole;
  Idx := Ord(TViews(vwPeriod));
  FPeriod[Idx].DayFrom := Frm.smdtpkFromPeriod;
  FPeriod[Idx].DayTo := Frm.smdtpkToPeriod;
  FPeriod[Idx].ButtonAply := Frm.glwbtnDatePeriod;
  FPeriod[Idx].ButtonAply.Enabled := False;

  FCommon[Idx].DisplayScale := Frm.advsedRowHeightPeriod;
  FCommon[Idx].InPositionGap := Frm.offchbInPositionGapPeriod;
  FCommon[Idx].NumberOfResource := Frm.advsedNumberOfResourcePeriod;
  FCommon[Idx].PropertiePanel := Frm.jstpnlPeriod;
  FCommon[Idx].ShowOccupied := Frm.offchbShowOccupiedPeriod;
  FCommon[Idx].SideBarPosition := Frm.offrgrSideBarPositionPeriod;
  FCommon[Idx].SideBarWidth := Frm.advsedSideBarWidthPeriod;
  FCommon[Idx].ColumnWidth := Frm.advsedColumnWidthPeriod;
  FCommon[Idx].HeaderHeight := Frm.advsedHeaderHeightPeriod;
end;

procedure TRolePlanner.FillPositionBar;
var
  Form: TfrmPlannerRole;
begin
  Form := frmPlannerRole;
  FPositionBar := TList<TAdvOfficeRadioGroup>.Create;
  FPositionBar.Add(Form.offrgrSideBarPositionMultiDay);
  FPositionBar.Add(Form.offrgrSideBarPositionPeriod);
  FPositionBar.Add(Form.offrgrSideBarPositionHalfDayPeriod);
  FPositionBar.Add(Form.offrgrSideBarPositionMonth);
  FPositionBar.Add(Form.offrgrSideBarPositionMultiMonth);
  FPositionBar.Add(Form.offrgrSideBarPositionTimeLine);
  FPositionBar.Add(Form.offrgrSideBarPositionActiveDay);
  FPositionBar.Add(Form.offrgrSideBarPositionActiveDayPeriod);
end;

procedure TRolePlanner.FillPropertiePanel;
var
  Form: TfrmPlannerRole;
begin
  Form := frmPlannerRole;
  FPropertiePanel[Ord(TViews(vwMultiDay))] := Form.jstpnlMultiDay;
  FPropertiePanel[Ord(TViews(vwPeriod))] := Form.jstpnlPeriod;
  FPropertiePanel[Ord(TViews(vwHalfDayPeriod))] := Form.jstpnlHalfDayPeriod;
  FPropertiePanel[Ord(TViews(vwMonth))] := Form.jstpnlMonth;
  FPropertiePanel[Ord(TViews(vwMultiMonth))] := Form.jstpnlMultiMonth;
  FPropertiePanel[Ord(TViews(vwTimeLine))] := Form.jstpnlTimeLine;
  FPropertiePanel[Ord(TViews(vwActiveDay))] := Form.jstpnlActiveDay;
  FPropertiePanel[Ord(TViews(vwActiveDayPeriod))] := Form.jstpnlActiveDayPeriod;

end;

procedure TRolePlanner.FillTimeLine;
var
  Idx: Integer;
  Frm: TfrmPlannerRole;
begin
  FTimes.Add(TTimes.Create);
  FPeriod.Add(TPeriod.Create);
  FOther.Add(nil);
  FCommon.Add(TCommon.Create);

  Frm := frmPlannerRole;
  Idx := Ord(TViews(vwTimeLine));
  FTimes[Idx].DisplayStart := Frm.dtpDisplayStartTimeLine;
  FTimes[Idx].DisplayEnd := Frm.dtpDisplayEndTimeLine;
  FTimes[Idx].ActiveStart := Frm.dtpActiveStartTimeLine;
  FTimes[Idx].ActiveEnd := Frm.dtpActiveEndTimeLine;
  FTimes[Idx].TimeLine := Frm.advcbxTimeLineTimeLine;
  FTimes[Idx].ButtonAply := Frm.glwbtnApplyTimeLine;
  FTimes[Idx].ButtonAply.Enabled := False;
  FTimes[Idx].OnlyActive := Frm.offchbOnlyActive;

  FPeriod[Idx].DayFrom := Frm.smdtpkFromTimeLine;
  FPeriod[Idx].DayTo := Frm.smdtpkToTimeLine;
  FPeriod[Idx].ButtonAply := Frm.glwbtnDateTimeLine;
  FPeriod[Idx].ButtonAply.Enabled := False;

  FCommon[Idx].DisplayScale := Frm.advsedRowHeightTimeLine;
  FCommon[Idx].InPositionGap := Frm.offchbInPositionGapTimeLine;
  FCommon[Idx].NumberOfResource := Frm.advsedNumberOfResourceTimeLine;
  FCommon[Idx].PropertiePanel := Frm.jstpnlTimeLine;
  FCommon[Idx].ShowOccupied := Frm.offchbShowOccupiedTimeLine;
  FCommon[Idx].SideBarPosition := Frm.offrgrSideBarPositionTimeLine;
  FCommon[Idx].SideBarWidth := Frm.advsedSideBarWidthTimeLine;
  FCommon[Idx].ColumnWidth := Frm.advsedColumnWidthTimeLine;
  FCommon[Idx].HeaderHeight := Frm.advsedHeaderHeightTimeLine;
end;

procedure TRolePlanner.GetDisplayUnit;
const
  Unit5 = 5;
  Twice = 2;
begin
  FDisplayUnit := TList<Integer>.Create;                             //  DisplayUnit
  FDisplayUnit.Add(Unit5);                                      // 5 min  -   5
  FDisplayUnit.Add(Unit5 + 1);                                  // 6 min  -   6
  FDisplayUnit.Add(Unit5 * Twice);                              // 10 min -  10
  FDisplayUnit.Add(FDisplayUnit.Last + Unit5);                  // 15 min -  15
  FDisplayUnit.Add(FDisplayUnit.Last * Twice);                  // 30 min -  30
  FDisplayUnit.Add(MinsPerHour);                                // 1 hod  -  60
  FDisplayUnit.Add(MinsPerHour * Twice);                        // 2 hod  - 120
  FDisplayUnit.Add(FDisplayUnit.Last + MinsPerHour);            // 3 hod  - 180
  FDisplayUnit.Add(FDisplayUnit.Last + MinsPerHour);            // 4 hod  - 240
  FDisplayUnit.Add(FDisplayUnit.Last + (MinsPerHour * Twice));  // 6 hod  - 360
  FDisplayUnit.Add(FDisplayUnit.Last + (MinsPerHour * Twice));  // 8 hod  - 480
end;

procedure TRolePlanner.GetHourMin(const HourMin: TList<Word>; const I: Integer);
begin
  HourMin.Add(HourOf(FTimes[I].DisplayStart.Time));
  HourMin.Add(MinuteOf(FTimes[I].DisplayStart.Time));
  HourMin.Add(HourOf(FTimes[I].DisplayEnd.Time));
  HourMin.Add(MinuteOf(FTimes[I].DisplayEnd.Time));
  HourMin.Add(HourOf(FTimes[I].ActiveStart.Time));
  HourMin.Add(MinuteOf(FTimes[I].ActiveStart.Time));
  HourMin.Add(HourOf(FTimes[I].ActiveEnd.Time));
  HourMin.Add(MinuteOf(FTimes[I].ActiveEnd.Time));
end;

procedure TRolePlanner.GetHours(var Hours: Integer; const I: Integer);
const
  Idx3 = 3;
  Idx4 = 4;
  Idx5 = 5;
  Hour6 = 6;
  Hour8 = 8;
begin
  Hours := 0;
  //1 hod 2 hod 3 hod 4 hod 6 hod 8 hod
  case FTimes[I].TimeLine.ItemIndex of
    0: Hours := 1;
    1..Idx3: Hours := FTimes[I].TimeLine.ItemIndex + 1;
    Idx4: Hours := Hour6;
    Idx5: Hours := Hour8;
  end;
end;

function TRolePlanner.GetItem: Boolean;
begin
  case TPage(FPager.ActivePageIndex) of
    pgDay: FItem := FPlannerDay.Items.Selected;
  else
    FItem := nil;
  end;

  Result := FItem <> nil;

  if Result then
    FDBKey := StrToInt(FItem.DBKey);
end;

procedure TRolePlanner.GetShapeDrawTool;
begin
  FShapeDrawTool := TObjectList<TShapeDrawTool>.Create(True);
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssCircle;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssTriangle;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssHArrow;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssVArrow;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssTriangles;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssCircles;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssSquares;
end;

procedure TRolePlanner.InitialTimeLineUnit;
const
  TimeUnitCount = 2;
var
  I: Integer;
begin
  FTimeLineUnit := TList<Integer>.Create;

  for I := 0 to TimeUnitCount do
    FTimeLineUnit.Add(-1);
end;

procedure TRolePlanner.ItemShape(const AShape: Byte);
begin
Exit;

  if not GetItem then
    Exit;

  FqryItemShape.ParamByName('IDPLANNERS').AsInteger := FDBKey;
  FqryItemShape.ParamByName('SHAPE').AsInteger := AShape;

  try
//    TFDFunction.QryExecute(FqryItemShape, esUpdate);
    FDataSet.Refresh;
  except
    raise;
  end;
end;

procedure TRolePlanner.RepeatSideBar(ARepeat: Boolean);
var
  Planner: TDBPlanner;
  Idx: Integer;
begin
  case TPage(FPager.ActivePageIndex) of
    pgDay:
    begin
      Planner := FPlannerDay;
      Idx := FcbxViews.ItemIndex;
    end;
  else
    Exit;
  end;

  Planner.Sidebar.ShowInPositionGap := ARepeat;
  Planner.Sidebar.Visible := not ARepeat;
  FCommon[Idx].SideBarPosition.Enabled := not ARepeat;

  if ARepeat then
    Planner.PositionGap := FCommon[Idx].SideBarWidth.Value
  else
    Planner.PositionGap := 0;
end;

procedure TRolePlanner.SetActiveDay;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBActiveDaySource;
begin
  Form := frmPlannerRole;
  SetDisplayStartEnd(FPlannerDay);
  FPlannerDay.Sidebar.Width := SideBar45;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightActiveDay.Value;

  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedActiveDay.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapActiveDay.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionActiveDay.ItemIndex);

  ItemSource := Form.dbacdysrActiveDay;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    Form.SetDislplayStartEnd(Form.advcbxTimeLineTimeLine.Tag, FDisplayUnit[Form.advcbxTimeLineTimeLine.ItemIndex]);
    ItemSource.Day := Form.smdtpkDayActiveDay.Date;
    ItemSource.NumberOfDays := Form.advsedNumberOfDayActiveDay.Value;
    RepeatSideBar(Form.offchbRepeatSideBarActiveDay.Checked);
  finally
    ItemSource.Planner.Update;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetActiveDayPeriod;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBActiveDayPeriodSource;
begin
  Form := frmPlannerRole;
  FPlannerDay.Sidebar.Width := SideBar65;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightActiveDayPeriod.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedActiveDayPeriod.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapActiveDayPeriod.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionActiveDayPeriod.ItemIndex);

  ItemSource := Form.dbacdyprsrActiveDayPeriod;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.StartDate := Form.smdtpkFromActiveDayPeriod.Date;
    ItemSource.EndDate := Form.smdtpkToActiveDayPeriod.Date;
  finally
    ItemSource.UpdatePlanner;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetCommon(const I: Integer);
var
  Planner: TDBPlanner;
begin
  if I = FPlanners then
    Planner := FPlannerWeek
  else
    Planner := FPlannerDay;

  if Assigned(FCommon[I].ShowOccupied) then
    Planner.Sidebar.ShowOccupied := FCommon[I].ShowOccupied.Checked;
  if Assigned(FCommon[I].InPositionGap) then
    Planner.ShowOccupiedInPositionGap := FCommon[I].InPositionGap.Checked;
  if Assigned(FCommon[I].SideBarPosition) then
    Planner.SideBar.Position := TSideBarPosition(FCommon[I].SideBarPosition.ItemIndex);
  if Assigned(FCommon[I].RepeatSideBar) then
    RepeatSideBar(FCommon[I].RepeatSideBar.Checked);

  if Assigned(FCommon[I].ColumnWidth) then
  begin
    if (FCommon[I].SideBarPosition.ItemIndex = SideBarTop) then
    begin
      Planner.Display.DisplayScale := FCommon[I].ColumnWidth.Value;
      Planner.SideBar.Width := FCommon[I].HeaderHeight.Value;
    end
    else
    begin
      Planner.Display.DisplayScale := 32; //FCommon[I].DisplayScale.Value;
      Planner.SideBar.Width := SideBar65; //FCommon[I].SideBarWidth.Value;
    end;

    FCommon[I].HeaderHeight.Visible := (FCommon[I].SideBarPosition.ItemIndex = SideBarTop);
    FCommon[I].ColumnWidth.Visible := FCommon[I].HeaderHeight.Visible;
    FCommon[I].SideBarWidth.Visible := not FCommon[I].HeaderHeight.Visible;
    FCommon[I].DisplayScale.Visible := not FCommon[I].HeaderHeight.Visible;
  end;

  SetNumberOfResources(I);
end;

procedure TRolePlanner.SetDisplayStartEnd(const Planner: TDBPlanner);
var
  DisplayEnd: Integer;
begin
  DisplayEnd := (Trunc(MinsPerHour / Planner.Display.DisplayUnit) * HoursPerDay) - 1;
  Planner.Display.DisplayStart := 0;
  Planner.Display.DisplayEnd := DisplayEnd;
end;

procedure TRolePlanner.SetHalfDayPeriod;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBHalfDayPeriodSource;
begin
  Form := frmPlannerRole;
  FPlannerDay.Sidebar.Width := SideBar65;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightHalfDayPeriod.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbInPositionGapHalfDayPeriod.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbShowOccupiedHalfDayPeriod.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionHalfDayPeriod.ItemIndex);

  ItemSource := Form.dbhldyprsrHalfDayPeriod;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.StartDate := Form.smdtpkFromHalfDayPeriod.Date;
    ItemSource.EndDate := Form.smdtpkToHalfDayPeriod.Date;
  finally
    ItemSource.UpdatePlanner;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetMonth;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBMonthSource;
begin
  Form := frmPlannerRole;
  FPlannerDay.Sidebar.Width := SideBar65;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightMonth.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedMonth.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapMonth.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionMonth.ItemIndex);

  ItemSource := Form.dbmnsrMonth;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.Month := Form.advcbxMonthMonth.ItemIndex + 1;
    ItemSource.Year := Form.advsedYearMonth.Value;
  finally
    ItemSource.Planner.Update;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetMultiDay;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBDaySource;
begin
  Form := frmPlannerRole;
  SetDisplayStartEnd(FPlannerDay);
  FPlannerDay.Sidebar.Width := SideBar45;

  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedMultiDay.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapMultiDay.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionMultiDay.ItemIndex);

  ItemSource := Form.dbdysrMultiDay;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    Form.SetDislplayStartEnd(Form.advcbxTimeLineMultiDay.Tag, FDisplayUnit[Form.advcbxTimeLineMultiDay.ItemIndex]);
    FPlannerDay.Display.DisplayScale := Form.advsedRowHeightMultiDay.Value;
    ItemSource.Day := Form.smdtpkDayMultiDay.Date;
    ItemSource.NumberOfDays := Form.advsedNumberOfDayMultiDay.Value;
    ItemSource.DayIncrement := Form.advsedDayIncrementMultiDay.Value;
    RepeatSideBar(Form.offchbRepeatSideBarMultiDay.Checked);
  finally
    ItemSource.Planner.Update;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetMultiMonth;
const
  SideBar20 = 20;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBMultiMonthSource;
begin
  Form := frmPlannerRole;
  FPlannerDay.Sidebar.Width := SideBar20;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightMultiMonth.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedMultiMonth.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapMultiMonth.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionMultiMonth.ItemIndex);

  ItemSource := Form.dbmlmnsrMultiMonth;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.StartMonth := Form.advcbxStartMonthMultiMonth.ItemIndex + 1;
    ItemSource.Year := Form.advsedYearMultiMonth.Value;
    ItemSource.NumberOfMonths := Form.advsedMonthOfMonthMultiMonth.Value;
    ItemSource.MonthIncrement := Form.advsedMonthIncrementMultiMonth.Value;
    RepeatSideBar(Form.offchbRepeatSideBarMultiMonth.Checked);
  finally
    ItemSource.UpdatePlanner;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetNumberOfResources(const I: Integer);
var
  NumOfResources: Integer;
begin
  if Assigned(FCommon[I].NumberOfResource) then
  begin
    NumOfResources := FCommon[I].NumberOfResource.Value;

    case TViews(I) of
      vwMultiDay: TDBDaySource(FPlannerDay.ItemSource).NumberOfResources := NumOfResources;
      vwPeriod: TDBPeriodSource(FPlannerDay.ItemSource).NumberOfResources := NumOfResources;
      vwHalfDayPeriod: TDBHalfDayPeriodSource(FPlannerDay.ItemSource).NumberOfResources := NumOfResources;
      vwMonth: TDBMonthSource(FPlannerDay.ItemSource).NumberOfResources := NumOfResources;
      vwTimeLine: TDBTimeLineSource(FPlannerDay.ItemSource).NumberOfResources := NumOfResources;
      vwActiveDay: TDBActiveDaySource(FPlannerDay.ItemSource).NumberOfResources := NumOfResources;
      vwActiveDayPeriod: TDBActiveDayPeriodSource(FPlannerDay.ItemSource).NumberOfResources := NumOfResources;
    end;
  end;
end;

procedure TRolePlanner.SetOther(const I: Integer);
begin
  if I = FPlanners then
  begin
    TDBWeekSource(FPlannerWeek.ItemSource).Month := FOther[I].Month.ItemIndex + 1;
    TDBWeekSource(FPlannerWeek.ItemSource).Year := FOther[I].Year.Value;
    TDBWeekSource(FPlannerWeek.ItemSource).Weeks := FOther[I].NumberOf.Value;
    TDBWeekSource(FPlannerWeek.ItemSource).WeekStart := FOther[I].Start.ItemIndex;
    //  Dni v ComboBox s v inom porad, ako to vyaduje DBPlanner
    FOther[I].Start.OnChange(FOther[I].Start);
  end
  else
  begin
    case TViews(I) of
      vwMultiDay:
      begin
        TDBDaySource(FPlannerDay.ItemSource).Day := FOther[I].Day.Date;
        TDBDaySource(FPlannerDay.ItemSource).NumberOfDays := FOther[I].NumberOf.Value;
        TDBDaySource(FPlannerDay.ItemSource).DayIncrement := FOther[I].Increment.Value;
      end;
      vwMonth:
      begin
        TDBMonthSource(FPlannerDay.ItemSource).Month := FOther[I].Month.ItemIndex + 1;
        TDBMonthSource(FPlannerDay.ItemSource).Year := FOther[I].Year.Value;
        TDBMonthSource(FPlannerDay.ItemSource).UpdatePlanner;
      end;
      vwMultiMonth:
      begin
        TDBMultiMonthSource(FPlannerDay.ItemSource).StartMonth := FOther[I].Month.ItemIndex + 1;
        TDBMultiMonthSource(FPlannerDay.ItemSource).Year := FOther[I].Year.Value;
        TDBMultiMonthSource(FPlannerDay.ItemSource).NumberOfMonths := FOther[I].NumberOf.Value;
        TDBMultiMonthSource(FPlannerDay.ItemSource).MonthIncrement := FOther[I].Increment.Value;
        TDBMultiMonthSource(FPlannerDay.ItemSource).UpdatePlanner;
      end;
      vwActiveDay:
      begin
        TDBActiveDaySource(FPlannerDay.ItemSource).Day := FOther[I].Day.Date;
        TDBActiveDaySource(FPlannerDay.ItemSource).NumberOfDays := FOther[I].NumberOf.Value;
      end;
    end;
  end;
end;

procedure TRolePlanner.SetPeriod(const I: Integer);
begin
  if not Assigned(FPeriod[I]) then
    Exit;

  case TViews(I) of
    vwPeriod:
    begin
      TDBPeriodSource(FPlannerDay.ItemSource).StartDate := FPeriod[I].DayFrom.Date;
      TDBPeriodSource(FPlannerDay.ItemSource).EndDate := FPeriod[I].DayTo.Date;
    end;
    vwHalfDayPeriod:
    begin
      TDBHalfDayPeriodSource(FPlannerDay.ItemSource).StartDate := FPeriod[I].DayFrom.Date;
      TDBHalfDayPeriodSource(FPlannerDay.ItemSource).EndDate := FPeriod[I].DayTo.Date;
    end;
    vwTimeLine:
    begin
      TDBTimeLineSource(FPlannerDay.ItemSource).StartDate := FPeriod[I].DayFrom.Date;
      TDBTimeLineSource(FPlannerDay.ItemSource).EndDate := FPeriod[I].DayTo.Date;
    end;
    vwActiveDayPeriod:
    begin
      TDBActiveDayPeriodSource(FPlannerDay.ItemSource).StartDate := FPeriod[I].DayFrom.Date;
      TDBActiveDayPeriodSource(FPlannerDay.ItemSource).EndDate := FPeriod[I].DayTo.Date;
    end;
  end;
end;

procedure TRolePlanner.SetPlannerProperies;
var
  I: Integer;
begin
  for I := 0 to FPlanners - 1 do
    FCommon[I].PropertiePanel.Visible := False;

  FPlannerDay.ItemSource.BeginUpdate;

  try
    SetCommon(FcbxViews.ItemIndex);
    SetOther(FcbxViews.ItemIndex);
    SetPeriod(FcbxViews.ItemIndex);
    SetTimes(FcbxViews.ItemIndex);
  finally
    FPlannerDay.Update;
    FPlannerDay.ItemSource.EndUpdate;
    FCommon[FcbxViews.ItemIndex].PropertiePanel.Visible := True;
  end;
end;

procedure TRolePlanner.SetTimeLine(Dsp: TPlannerDisplay; const Hours, I: Integer);
begin
  FPlannerDay.Mode.TimeLineStart := FPeriod[I].DayFrom.Date;
  Dsp.DisplayUnit := Hours * MinsPerHour;
  Dsp.DisplayStart := 0;
  //  Poet dn * poet blokov -1
  Dsp.DisplayEnd := ((DaysBetween(FPeriod[I].DayFrom.Date, FPeriod[I].DayTo.Date) + 1) * Trunc(HoursPerDay/Hours)) -1;

  if FTimes[I].OnlyActive.Checked then
  begin
    //  Poet skrytch blokov
    FPlannerDay.Mode.TimeLineNVUBegin :=
      Trunc(HoursBetween(FTimes[I].ActiveStart.Time, FTimes[I].DisplayStart.Time) / Hours);
    FPlannerDay.Mode.TimeLineNVUEnd := Trunc(HoursBetween(FTimes[I].DisplayEnd.Time, FTimes[I].ActiveEnd.Time) / Hours);
  end
  else
  begin
    FPlannerDay.Mode.TimeLineNVUBegin := 0;
    FPlannerDay.Mode.TimeLineNVUEnd := 0;
  end;
end;

procedure TRolePlanner.SetTimes(const I: Integer);
const
  DspStrH = 0;  DspStrM = 1;  DspEndH = 2;  DspEndM = 3;  ActStrH = 4;  ActStrM = 5;  ActEndH = 6;  ActEndM = 7;
var
  Dsp: TPlannerDisplay;
  RowsPerHour, DspStart, Hours: Integer;
  HourMin: TList<Word>;
begin
  if not Assigned(FTimes[I]) then
  begin
    FPlannerDay.Mode.TimeLineNVUBegin := 0;
    FPlannerDay.Mode.TimeLineNVUEnd := 0;
    Exit;
  end;

  Dsp := FPlannerDay.Display;

  case TViews(I) of
    vwMultiDay, vwActiveDay: Dsp.DisplayUnit := FDisplayUnit[FTimes[I].TimeLine.ItemIndex];
    vwTimeLine:
    begin
      GetHours(Hours, I);
      SetTimeLine(Dsp, Hours, I);
      Exit;
    end;
  end;

  RowsPerHour := (MinsPerHour div Dsp.DisplayUnit);
  HourMin := TList<Word>.Create;

  try
    GetHourMin(HourMin, I);
    DspStart := HourMin[DspStrH] * RowsPerHour + (HourMin[DspStrM] div Dsp.DisplayUnit);

    if not (TViews(I) = vwTimeLine) then
    begin
      Dsp.DisplayStart := DspStart;
      Dsp.DisplayEnd := (HourMin[DspEndH] * RowsPerHour) + (HourMin[DspEndM] div Dsp.DisplayUnit);
    end;

    Dsp.ActiveStart := (HourMin[ActStrH] * RowsPerHour + (HourMin[ActStrM] div Dsp.DisplayUnit)) - DspStart;
    Dsp.ActiveEnd := (HourMin[ActEndH] * RowsPerHour + (HourMin[ActEndM] div Dsp.DisplayUnit)) - DspStart;
  finally
    FreeAndNil(HourMin);
  end;
end;

end.


